<?php

namespace Drupal\imgproxy;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Url;
use Drupal\imgproxy\Imgproxy\Gravity;
use Drupal\imgproxy\Imgproxy\ImgProxyUrl;
use Drupal\imgproxy\Imgproxy\ResizeType;
use Drupal\imgproxy\Imgproxy\Watermark;
use Drupal\imgproxy\Imgproxy\WatermarkPosition;

/**
 * Main service for working with imgproxy.
 */
class ImgProxy {

  /**
   * Drupal config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * ImgProxy constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   Config factory injection.
   */
  public function __construct(ConfigFactoryInterface $config) {
    $this->config = $config->get('imgproxy.settings');
  }

  /**
   * Creates imgproxy url for given data.
   *
   * @param string|\Drupal\Core\Url $url
   *   Source image url. Must be accessible from imgproxy server.
   * @param array $settings
   *   Processing settings.
   *
   * @return string
   *   Processed image url for the given parameters.
   *
   * @see https://github.com/imgproxy/imgproxy/blob/master/docs/generating_the_url_advanced.md
   *
   * @todo More description for settings.
   */
  public function getUrl($url, array $settings) {
    if ($url instanceof Url) {
      // Twig should not modify the original Url object.
      $url = Url::fromUri($url)->setAbsolute()->toString();
    }
    $url = $this->config->get('local', FALSE)
      ? str_replace(['public:', 'private:'], 'local:/', $url)
      : $url;
    $instance = (new ImgProxyUrl())
      ->setImageUrl($url)
      ->setEndpoint($this->config->get('endpoint'))
      ->setKey($this->config->get('key'))
      ->setSalt($this->config->get('salt'))
      ->setResize(
        isset($settings['resize']) ? new ResizeType($settings['resize']) : NULL
      )
      ->setGravity(
        isset($settings['gravity']) ? new Gravity($settings['gravity']) : NULL
      )
      ->setEnlarge($settings['enlarge'] ?? NULL)
      ->setWidth($settings['width'] ?? NULL)
      ->setHeight($settings['height'] ?? NULL)
      ->setBackground($settings['bg'] ?? NULL)
      ->setBlur($settings['blur'] ?? NULL)
      ->setDpr($settings['dpr'] ?? NULL)
      ->setExtend($settings['extend'] ?? NULL)
      ->setExtension($settings['extension'] ?? NULL)
      ->setQuality($settings['quality'] ?? NULL)
      ->setSharpen($settings['sharpen'] ?? NULL);
    if ($settings['watermark'] && $settings['watermark']['opacity']) {
      $instance->setWatermark(
        Watermark::create(
          $settings['watermark']['opacity'],
          isset($settings['watermark']['position'])
            ? new WatermarkPosition($settings['watermark']['position'])
            : NULL,
          $settings['watermark']['x'] ?? NULL,
          $settings['watermark']['y'] ?? NULL,
          $settings['watermark']['scale'] ?? NULL
        )
      );
      $instance->setCachebuster($settings['cachebuster']);
    }
    return $instance->toString();
  }

}
